use std::net::TcpStream;
use std::io::prelude::*;
use std::thread;
use std::sync::{Arc, Mutex, RwLock};
use libc::splice;
use libc::pipe;
use libc::{SPLICE_F_MORE,SPLICE_F_MOVE};
use std::os::unix::io::AsRawFd;
use std::ptr;
use std::time::SystemTime;

use crate::protocol;
pub mod guard;

pub struct Client<'a>{
    client: Arc<Mutex<TcpStream>>,
    server:Arc<Mutex<TcpStream>>,
    hs: protocol::HandShake<'a>,
    run : Arc<RwLock<bool>>,
}

impl<'a> Client<'a> {
    pub fn new(client: TcpStream, server: TcpStream, handshake: protocol::HandShake) -> Client{
        Client {
            client: Arc::new(Mutex::new(client)),
            server: Arc::new(Mutex::new(server)),
            hs: handshake,
            run: Arc::new(RwLock::new(true)),
            //threads: None,
        }
    }

    pub fn to_string(&self){
        println!("len_pack {}", self.hs.get_host_name());
    }

    fn join_conexions_mutex(c1: Arc<Mutex<TcpStream>>,
            c2: Arc<Mutex<TcpStream>>,
            run: Arc<RwLock<bool>>){
        let mut buf: [u8; 200] = [0; 200];
        let mut client = c1.lock().unwrap().try_clone().unwrap();
        let mut len2;
        let mut p: [i32;2] = [0,0];
        let tam = 200;
        let mut max = 0;
        let mut time;
        let mut time2;
        let mut fin = 0;
        let mut fin2 = 0;
        unsafe{len2 = pipe(p.as_ptr() as *mut i32);}
        while *run.read().unwrap() {
            time = SystemTime::now();
            let res=client.read(&mut buf);
            match res {
                Ok(leng) => {
                    if leng == 0 {
                        *run.write().unwrap()=false;
                    }
                    match c2.lock().unwrap().write(&buf [.. leng]) {
                        Ok(_l) => {
                            time2 = SystemTime::now();
                            fin2 = time2.duration_since(time).unwrap_or_default().as_micros();
                            if fin2 > fin {
                                fin = fin2;
                            }
                        },
                        Err(_e) => *run.write().unwrap()=false,
                    }
                },
                Err(_e) => *run.write().unwrap()=false,
            }

        }
        println!("time {}", fin);
    }

    fn join_conexions_kernel(c1: Arc<Mutex<TcpStream>>,
            c2: Arc<Mutex<TcpStream>>,
            run: Arc<RwLock<bool>>){
        let mut client = c1.lock().unwrap().try_clone().unwrap();
        let mut len2;
        let mut p: [i32;2] = [0,0];
        let tam = 20000;
        let mut len;
        let mut time;
        let mut time2;
        let mut fin = 0;
        let mut fin2 = 0;
        unsafe{len2 = pipe(p.as_ptr() as *mut i32);}
        while *run.read().unwrap() {
            time = SystemTime::now();
            unsafe {
                len = splice(client.as_raw_fd(), ptr::null_mut(),
                        p[1], ptr::null_mut(), tam, SPLICE_F_MOVE);

                if len <= 0 {
                    *run.write().unwrap()=false;
                    break;
                }

                splice(p[0], ptr::null_mut(),
                        c2.lock().unwrap().as_raw_fd(), ptr::null_mut(),
                        len as usize, SPLICE_F_MOVE | SPLICE_F_MORE);
                time2 = SystemTime::now();
                fin2 = time2.duration_since(time).unwrap_or_default().as_micros();
                if fin2 > fin {
                    fin = fin2;
                }
            }

        }
        println!("time {}",fin);
    }

    pub fn start_proxy(&self) -> (thread::JoinHandle<()>, thread::JoinHandle<()>) {

        let c1 = self.client.clone();
        let s1 = self.server.clone();
        let r1 = self.run.clone();
        let handler1 = thread::spawn( || {
            Self::join_conexions_mutex(s1,
                c1,
                r1)}
        );

        let c2 = self.client.clone();
        let s2 = self.server.clone();
        let r2 = self.run.clone();
        let handler2 = thread::spawn( || {
            Self::join_conexions_mutex(c2,
                s2,
                r2)}
        );

        return (handler1, handler2);
    }
}
